"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const QueryBuilderHelper_1 = require("./QueryBuilderHelper");
class SmartQueryHelper {
    static processParams(params, field) {
        if (__1.Utils.isEntity(params)) {
            return SmartQueryHelper.processEntity(params, field);
        }
        if (params === undefined) {
            return null;
        }
        if (Array.isArray(params)) {
            return params.map(item => SmartQueryHelper.processParams(item, field));
        }
        if (__1.Utils.isObject(params)) {
            Object.keys(params).forEach(k => {
                params[k] = SmartQueryHelper.processParams(params[k], k);
            });
        }
        return params;
    }
    static processWhere(where) {
        where = this.processParams(where);
        if (!__1.Utils.isObject(where)) {
            return where;
        }
        Object.entries(where).map(([key, value]) => {
            if (QueryBuilderHelper_1.QueryBuilderHelper.GROUP_OPERATORS[key]) {
                return value.map((sub) => this.processWhere(sub));
            }
            if (!SmartQueryHelper.isSupported(key)) {
                return where;
            }
            delete where[key];
            if (key.includes(':')) {
                const [k, expr] = key.split(':');
                where[k] = SmartQueryHelper.processExpression(expr, value);
            }
            else {
                const m = key.match(/([\w-]+) ?([<>=!]+)$/);
                where[m[1]] = SmartQueryHelper.processExpression(m[2], value);
            }
        });
        return where;
    }
    static processEntity(entity, field) {
        if (field) {
            return entity.__primaryKey;
        }
        return { [entity.__primaryKeyField]: entity.__primaryKey };
    }
    static processExpression(expr, value) {
        switch (expr) {
            case '>': return { '$gt': value };
            case '<': return { '$lt': value };
            case '>=': return { '$gte': value };
            case '<=': return { '$lte': value };
            case '!=': return { '$ne': value };
            case '!': return { '$not': value };
            default: return { ['$' + expr]: value };
        }
    }
    static isSupported(key) {
        return !!SmartQueryHelper.SUPPORTED_OPERATORS.find(op => key.includes(op));
    }
}
SmartQueryHelper.SUPPORTED_OPERATORS = ['>', '<', '<=', '>=', '!', '!=', ':in', ':nin', ':gt', ':gte', ':lt', ':lte', ':ne', ':not'];
exports.SmartQueryHelper = SmartQueryHelper;
